@@ -41,42 +41,8 @@ class XmlResourceXPathParser(XPath1Parser):
 XmlResourceXPathParser.build_tokenizer()
 
 
-def is_remote_url(url):
-    return isinstance(url, str) and urlsplit(url).scheme not in ('', 'file')
-
-
-def url_path_is_directory(url):
-    return os.path.isdir(urlsplit(url).path)
-
-
-def url_path_is_file(url):
-    return os.path.isfile(urlsplit(url).path)
-
-
-def update_prefix(namespaces, prefix, uri):
-    """Update namespace registration without overwrite an existing one."""
-    if not prefix:
-        if '' not in namespaces:
-            namespaces[prefix] = uri
-            return
-        elif namespaces[''] == uri:
-            return
-        prefix = 'default'
-
-    while prefix in namespaces:
-        if namespaces[prefix] == uri:
-            return
-        match = re.search(r'(\d+)$', prefix)
-        if match:
-            index = int(match.group()) + 1
-            prefix = prefix[:match.span()[0]] + str(index)
-        else:
-            prefix += '0'
-    namespaces[prefix] = uri
-
-
 ###
-# API for XML resources
+# URL normalization (that fixes many headaches :)
 
 def normalize_url(url, base_url=None, keep_relative=False):
     """
@@ -167,6 +133,65 @@ def normalize_url(url, base_url=None, keep_relative=False):
     return filter_url(normalized_url)
 
 
+###
+# Internal helper functions
+
+def is_url(obj):
+    """
+    Checks if and object can be an URL, restricting to strings that cannot be XML data.
+    """
+    if not isinstance(obj, (str, bytes)):
+        return False
+    elif '\n' in obj or obj.lstrip().startswith('<'):
+        return False
+
+    try:
+        urlsplit(obj.strip())
+    except ValueError:
+        return False
+    else:
+        return True
+
+
+def is_remote_url(url):
+    return is_url(url) and urlsplit(normalize_url(url)).scheme not in ('', 'file')
+
+
+def is_local_url(url):
+    return is_url(url) and urlsplit(normalize_url(url)).scheme in ('', 'file')
+
+
+def url_path_is_file(url):
+    if not is_local_url(url):
+        return False
+    return os.path.isfile(url) or os.path.isfile(urlsplit(normalize_url(url)).path)
+
+
+def update_prefix(namespaces, prefix, uri):
+    """Update namespace registration without overwrite an existing one."""
+    if not prefix:
+        if '' not in namespaces:
+            namespaces[prefix] = uri
+            return
+        elif namespaces[''] == uri:
+            return
+        prefix = 'default'
+
+    while prefix in namespaces:
+        if namespaces[prefix] == uri:
+            return
+        match = re.search(r'(\d+)$', prefix)
+        if match:
+            index = int(match.group()) + 1
+            prefix = prefix[:match.span()[0]] + str(index)
+        else:
+            prefix += '0'
+    namespaces[prefix] = uri
+
+
+###
+# API for XML resources
+
 def normalize_locations(locations, base_url=None, keep_relative=False):
     """
     Returns a list of normalized locations. The locations are normalized using
@@ -343,6 +368,10 @@ class XMLResource(object):
             elif value not in SECURITY_MODES:
                 msg = "'allow' attribute: {!r} is not a security mode"
                 raise XMLSchemaValueError(msg.format(value))
+            elif value == 'sandbox' and self._base_url is None:
+                raise XMLSchemaResourceError(
+                    "block access to files out of sandbox requires 'base_url' to be set"
+                )
         elif name == 'defuse':
             if not isinstance(value, str):
                 msg = "invalid type {!r} for the attribute 'defuse'"
@@ -372,24 +401,28 @@ class XMLResource(object):
         if self.allow == 'all':
             return
         elif self.allow == 'remote':
-            if is_remote_url(url):
-                return
-            raise XMLSchemaResourceError("block access to local resource {}".format(url))
+            if is_local_url(url):
+                raise XMLSchemaResourceError("block access to local resource {}".format(url))
         elif is_remote_url(url):
             raise XMLSchemaResourceError("block access to remote resource {}".format(url))
-        elif self.allow == 'local' or self._base_url is None:
-            return
-        else:
-            path = os.path.normpath(os.path.normcase(urlsplit(url).path))
-            base_path = os.path.normpath(os.path.normcase(urlsplit(self._base_url).path))
-            if not path.startswith(base_path):
-                raise XMLSchemaResourceError("block access to out of sandbox file {}".format(path))
+        elif self.allow == 'sandbox':
+            if not url.startswith(normalize_url(self._base_url)):
+                raise XMLSchemaResourceError("block access to out of sandbox file {}".format(url))
 
     def _fromsource(self, source):
-        url = None
-        if hasattr(source, 'tag') and hasattr(source, 'attrib'):
-            self._lazy = False
-            return source, None, None  # Source is already an Element --> nothing to load
+        if is_url(source):
+            url = normalize_url(source)
+            self._access_control(url)
+            _url, self._url = self._url, url
+            try:
+                with urlopen(url, timeout=self.timeout) as resource:
+                    if self._lazy:
+                        for _, root in self.iterparse(resource, events=('start',)):
+                            return root, None, url
+                    else:
+                        return self.parse(resource).getroot(), None, url
+            finally:
+                self._url = _url
 
         elif isinstance(source, str):
             _url, self._url = self._url, None
@@ -400,10 +433,6 @@ class XMLResource(object):
                         return root, source, None
                 else:
                     return self.fromstring(source), source, None
-            except (ElementTree.ParseError, PyElementTree.ParseError, UnicodeEncodeError):
-                if '\n' in source or source.lstrip().startswith('<'):
-                    raise
-                url = normalize_url(source)
             finally:
                 self._url = _url
 
@@ -424,8 +453,10 @@ class XMLResource(object):
                 if is_remote_url(source.url):
                     url = source.url
                     self._access_control(url)
+                else:
+                    url = None
             except AttributeError:
-                pass
+                url = None
 
             _url, self._url = self._url, url
             try:
@@ -437,6 +468,10 @@ class XMLResource(object):
             finally:
                 self._url = _url
 
+        elif hasattr(source, 'tag') and hasattr(source, 'attrib'):
+            self._lazy = False
+            return source, None, None  # Source is already an Element --> nothing to load
+
         else:
             # Try ElementTree object at last
             try:
@@ -448,24 +483,11 @@ class XMLResource(object):
                     self._lazy = False
                     return root, None, None
 
-        if url is None:
             raise XMLSchemaTypeError(
                 "wrong type %r for 'source' attribute: an ElementTree object or "
                 "an Element instance or a string containing XML data or an URL "
                 "or a file-like object is required." % type(source)
             )
-        else:
-            self._access_control(url)
-            _url, self._url = self._url, url
-            try:
-                with urlopen(url, timeout=self.timeout) as resource:
-                    if self._lazy:
-                        for _, root in self.iterparse(resource, events=('start',)):
-                            return root, None, url
-                    else:
-                        return self.parse(resource).getroot(), None, url
-            finally:
-                self._url = _url
 
     @property
     def root(self):
