@@ -26,7 +26,7 @@ from xmlschema.etree import etree_tostring, ElementTree, lxml_etree, \
 from xmlschema.helpers import iter_nested_items
 from xmlschema.resources import fetch_namespaces
 from xmlschema.xpath import XMLSchemaContext
-from xmlschema.validators import XsdValidator, XsdType, Xsd11ComplexType, XsdAttributeGroup
+from xmlschema.validators import XsdValidator, XsdType, Xsd11ComplexType
 
 from .case_class import XsdValidatorTestCase
 from .observers import SchemaObserver
@@ -78,8 +78,7 @@ def make_schema_test_class(test_file, test_args, test_num, schema_class, check_w
                 components_ids = set([id(c) for c in xs.maps.iter_components()])
                 components_ids.update(id(c) for c in xs.meta_schema.iter_components())
                 missing = [
-                    c for c in SchemaObserver.components
-                    if id(c) not in components_ids and (not isinstance(c, XsdAttributeGroup) or c)
+                    c for c in SchemaObserver.components if id(c) not in components_ids
                 ]
                 if missing:
                     raise ValueError("schema missing %d components: %r" % (len(missing), missing))
@@ -107,7 +106,8 @@ def make_schema_test_class(test_file, test_args, test_num, schema_class, check_w
                 context = XMLSchemaContext(xs)
                 elements = [x for x in xs.iter()]  # Contains schema elements only
                 context_elements = [x for x in context.iter() if isinstance(x, XsdValidator)]
-                self.assertEqual(context_elements, [x for x in context.iter_descendants()])
+                descendants = [x for x in context.iter_descendants('descendant-or-self')]
+                self.assertTrue(x in descendants for x in context_elements)
                 for e in elements:
                     # Context elements can include elements of other schemas (by element ref)
                     self.assertIn(e, context_elements)
