@@ -16,10 +16,10 @@ from functools import lru_cache
 
 from ..exceptions import XMLSchemaKeyError, XMLSchemaTypeError, XMLSchemaValueError, \
     XMLSchemaWarning
-from ..namespaces import XSD_NAMESPACE, LOCATION_HINTS, NamespaceResourcesMap
+from ..namespaces import XSD_NAMESPACE, NamespaceResourcesMap
 from ..qnames import XSD_OVERRIDE, XSD_NOTATION, XSD_ANY_TYPE, XSD_SIMPLE_TYPE, \
     XSD_COMPLEX_TYPE, XSD_GROUP, XSD_ATTRIBUTE, XSD_ATTRIBUTE_GROUP, XSD_ELEMENT, \
-    XSI_TYPE, get_qname, local_name, qname_to_extended, is_xsd_redefine_or_override
+    XSI_TYPE, get_qname, local_name, get_extended_qname, is_xsd_redefine_or_override
 
 from . import XMLSchemaNotBuiltError, XMLSchemaModelError, XMLSchemaModelDepthError, \
     XsdValidator, XsdComponent, XsdAttribute, XsdSimpleType, XsdComplexType, \
@@ -90,7 +90,11 @@ def create_load_function(tag):
                                 break
 
             if elem.tag == XSD_OVERRIDE:
-                xsd_globals[qname] = (child, schema)
+                # Components which match nothing in the target schema are ignored. See the
+                # period starting with "Source declarations not present in the target set"
+                # of the paragraph https://www.w3.org/TR/xmlschema11-1/#override-schema.
+                if qname in xsd_globals:
+                    xsd_globals[qname] = (child, schema)
             else:
                 # Append to a list if it's a redefine
                 try:
@@ -299,7 +303,7 @@ class XsdGlobals(XsdValidator):
         if base_type.is_complex() and XSI_TYPE in base_type.attributes:
             base_type.attributes[XSI_TYPE].validate(type_name)
 
-        extended_name = qname_to_extended(type_name, namespaces)
+        extended_name = get_extended_qname(type_name, namespaces)
         xsi_type = lookup_type(extended_name, self.types, self.validator.BUILDERS_MAP)
         if xsi_type.is_derived(base_type):
             return xsi_type
@@ -432,8 +436,8 @@ class XsdGlobals(XsdValidator):
                     return True
 
         # Try from library location hint, if there is any.
-        if namespace in LOCATION_HINTS:
-            url = LOCATION_HINTS[namespace]
+        if namespace in self.validator.fallback_locations:
+            url = self.validator.fallback_locations[namespace]
             if url not in self.missing_locations:
                 try:
                     if self.validator.import_schema(namespace, url) is not None:
