@@ -11,16 +11,18 @@
 This module contains classes for other XML Schema identity constraints.
 """
 import re
+import math
 from collections import Counter
 from elementpath import XPath2Parser, ElementPathError, XPathContext
 
-from ..exceptions import XMLSchemaValueError
+from ..exceptions import XMLSchemaTypeError, XMLSchemaValueError
 from ..qnames import XSD_ANNOTATION, XSD_QNAME, XSD_UNIQUE, XSD_KEY, XSD_KEYREF, \
-    XSD_SELECTOR, XSD_FIELD, get_qname, qname_to_extended, is_not_xsd_annotation
+    XSD_SELECTOR, XSD_FIELD, get_qname, get_extended_qname, is_not_xsd_annotation
 from ..regex import get_python_regex
 from ..xpath import iter_schema_nodes
 from .exceptions import XMLSchemaValidationError
 from .xsdbase import XsdComponent
+from .attributes import XsdAttribute
 
 QNAME_PATTERN = re.compile(
     r'(?:(?P<prefix>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*):)?'
@@ -196,11 +198,24 @@ class XsdIdentity(XsdComponent):
                 self.fields = ref.fields
                 self.ref = ref
 
-        context = IdentityXPathContext(self.parent)
+        context = IdentityXPathContext(self.schema, item=self.parent)
+
         self.elements = {
             e: None for e in self.selector.token.select_results(context) if e.name
         }
 
+        if any(isinstance(e, XsdAttribute) for e in self.elements):
+            self.parse_error("selector xpath cannot select attributes")
+        elif not self.elements:
+            # Try to detect target XSD elements extracting QNames
+            # of the leaf elements from the XPath expression and
+            # use them to match global elements.
+
+            for qname in self.selector.token.iter_leaf_elements():
+                xsd_element = self.maps.elements.get(get_extended_qname(qname, self.namespaces))
+                if xsd_element is not None and xsd_element not in self.elements:
+                    self.elements[xsd_element] = None
+
     @property
     def built(self):
         return self.elements is not None
@@ -227,16 +242,19 @@ class XsdIdentity(XsdComponent):
                     value = decoders[k].value_constraint
                     if value is not None:
                         if decoders[k].type.root_type.name == XSD_QNAME:
-                            value = qname_to_extended(value, namespaces)
+                            value = get_extended_qname(value, namespaces)
 
                         if isinstance(value, list):
                             fields.append(tuple(value))
-                        elif isinstance(value, (bool, float)):
-                            fields.append(tuple([value, type(value)]))
-                        else:
+                        elif isinstance(value, bool):
+                            fields.append((value, bool))
+                        elif not isinstance(value, float):
                             fields.append(value)
+                        elif math.isnan(value):
+                            fields.append(('nan', float))
+                        else:
+                            fields.append((value, float))
 
-                        result.append(value)
                         continue
 
                 if not isinstance(self, XsdKey) or 'ref' in elem.attrib and \
@@ -251,16 +269,23 @@ class XsdIdentity(XsdComponent):
                 if decoders is None or decoders[k] is None:
                     fields.append(result[0])
                 else:
+                    if decoders[k].type.content_type_label not in ('simple', 'mixed'):
+                        raise XMLSchemaTypeError("%r field doesn't have a simple type!" % field)
+
                     value = decoders[k].data_value(result[0])
                     if decoders[k].type.root_type.name == XSD_QNAME:
-                        value = qname_to_extended(value, namespaces)
+                        value = get_extended_qname(value, namespaces)
 
                     if isinstance(value, list):
                         fields.append(tuple(value))
-                    elif isinstance(value, (bool, float)):
-                        fields.append(tuple([value, type(value)]))
-                    else:
+                    elif isinstance(value, bool):
+                        fields.append((value, bool))
+                    elif not isinstance(value, float):
                         fields.append(value)
+                    elif math.isnan(value):
+                        fields.append(('nan', float))
+                    else:
+                        fields.append((value, float))
             else:
                 raise XMLSchemaValueError("%r field selects multiple values!" % field)
 
