@@ -18,28 +18,28 @@ import threading
 from elementpath import AttributeNode, TypedElement, XPath2Parser, \
     XPathSchemaContext, AbstractSchemaProxy
 
-from .qnames import XSD_SCHEMA
+from .qnames import XSD_ANY_TYPE
 from .namespaces import XSD_NAMESPACE
 from .exceptions import XMLSchemaValueError, XMLSchemaTypeError
 
-_REGEX_TAG_POSITION = re.compile(r'\b\[\d+\]')
+_REGEX_TAG_POSITION = re.compile(r'\b\[\d+]')
 
 
-def iter_schema_nodes(elem, with_root=True, with_attributes=False):
+def iter_schema_nodes(root, with_root=True, with_attributes=False):
     """
     Iteration function for schema nodes. It doesn't yield text nodes,
     that are always `None` for schema elements, and detects visited
     element in order to skip already visited nodes.
 
-    :param elem: schema or schema's element.
+    :param root: schema or schema's element.
     :param with_root: if `True` yields initial element.
     :param with_attributes: if `True` yields also attribute nodes.
     """
     def attribute_node(x):
         return AttributeNode(*x)
 
-    def _iter_schema_nodes(_elem):
-        for child in _elem:
+    def _iter_schema_nodes(elem):
+        for child in elem:
             if child in nodes:
                 continue
             elif child.ref is not None:
@@ -58,15 +58,15 @@ def iter_schema_nodes(elem, with_root=True, with_attributes=False):
                     yield from map(attribute_node, child.attributes.items())
                 yield from _iter_schema_nodes(child)
 
-    if isinstance(elem, TypedElement):
-        elem = elem.elem
+    if isinstance(root, TypedElement):
+        root = root.elem
 
-    nodes = {elem}
+    nodes = {root}
     if with_root:
-        yield elem
-        if with_attributes:
-            yield from map(attribute_node, elem.attributes.items())
-    yield from _iter_schema_nodes(elem)
+        yield root
+    if with_attributes:
+        yield from map(attribute_node, root.attributes.items())
+    yield from _iter_schema_nodes(root)
 
 
 class XMLSchemaContext(XPathSchemaContext):
@@ -89,6 +89,10 @@ class XMLSchemaProxy(AbstractSchemaProxy):
             except AttributeError:
                 raise XMLSchemaTypeError("%r is not an XsdElement" % base_element)
 
+    @property
+    def xsd_version(self):
+        return self._schema.XSD_VERSION
+
     def bind_parser(self, parser):
         if parser.schema is not self:
             parser.schema = self
@@ -103,7 +107,11 @@ class XMLSchemaProxy(AbstractSchemaProxy):
         parser.tokenizer = parser.create_tokenizer(parser.symbol_table)
 
     def get_context(self):
-        return XMLSchemaContext(root=self._schema, item=self._base_element)
+        return XMLSchemaContext(
+            root=self._schema,
+            namespaces=self._schema.namespaces,
+            item=self._base_element
+        )
 
     def get_type(self, qname):
         try:
@@ -151,8 +159,8 @@ class XMLSchemaProxy(AbstractSchemaProxy):
                 yield xsd_type
 
     def get_primitive_type(self, xsd_type):
-        if not xsd_type.is_simple():
-            return self._schema.maps.types['{%s}anyType' % XSD_NAMESPACE]
+        if xsd_type.is_complex():
+            return self._schema.maps.types[XSD_ANY_TYPE]
         elif not hasattr(xsd_type, 'primitive_type'):
             if xsd_type.base_type is None:
                 return xsd_type
@@ -245,10 +253,7 @@ class ElementPathMixin(Sequence):
         return xpath_namespaces
 
     def _xpath_parse(self, path, namespaces=None):
-        path = path.strip()
-        if path.startswith('/') and not path.startswith('//'):
-            path = ''.join(['/', XSD_SCHEMA, path])
-        path = _REGEX_TAG_POSITION.sub('', path)  # Strips tags's positions from path
+        path = _REGEX_TAG_POSITION.sub('', path.strip())  # Strips tags positions from path
 
         namespaces = self._get_xpath_namespaces(namespaces)
         with self._xpath_lock:
